/* Copyright (c) 1989 by Carrick Sean Casey. All rights reserved. */

#include <sys/types.h>

/* macros */

/* this gets used a lot */
#define setlen (*pp = (unsigned char) (strlen(pbuf) + 1))

/* limits */

#define MAX_FIELDS	20	/* fields in a packet */
#define MAX_GROUPLEN	8	/* chars in a group name */
#define MAX_GROUPUSERS	16	/* people in the same group */
#define MAX_IDLEN	12	/* chars in a login id */
#define MAX_INPUTSTR	(250 - MAX_NICKLEN - MAX_NICKLEN - 6) /* input line */
#define MAX_INVITES	10	/* invitations in a group */
#define MAX_NICKLEN	12	/* chars in a username */
#define MAX_NODELEN	64	/* chars in a node name */
#define MAX_NOPONG	600	/* seconds a client may not PONG a PING */
#define MAX_PASSWDLEN	12	/* chars in a user password */
#define MAX_REALLEN	64	/* chars in a real life name */
#define MAX_TEXTLEN	(80 - MAX_NICKLEN - 5) 	/* max chars in a message */
#define MAX_TOPICLEN	32	/* chars in a group topic */

#define IDLE_POLLWAIT	30000	/* polltimeout (msecs) when users = 0 */
#define BUSY_POLLWAIT	10000	/* polltimeout (msecs) when users >= 1 */
#define POLL_DELAY	1000	/* min wait through each poll */

/* message types */
/* note: changing these necessitates upgrading the protocol number */

#define M_LOGIN		'a'	/* login packet */
#define M_LOGINOK	'a'	/* login packet */
#define M_OPEN		'b'	/* open msg to group */
#define M_PERSONAL	'c'	/* personal msg */
#define M_STATUS	'd'	/* status update message */
#define M_ERROR		'e'	/* error message */
#define M_IMPORTANT	'f'	/* special important announcement */
#define M_EXIT		'g'	/* tell other side to exit */
#define M_COMMAND	'h'	/* send a command from user */
#define M_CMDOUT	'i'	/* output from a command */
#define M_PROTO		'j'	/* protocol version information */
#define M_BEEP		'k'	/* beeps */
#define M_PING		'l'	/* ping packet */
#define M_PONG		'm'	/* return for ping packet */

/* return codes for visible() */

#define V_NODETECT	1	/* group cannot be detected */
#define V_DETECT	2	/* group can be detected but not named */
#define V_VISIBLE	3	/* group is fully visible */

/* misc */

#define CMDCHAR		'/'	/* char for prefixing commands in client */

/* structs */


struct Namelist {
	char name[MAX_NICKLEN+1];
	struct Namelist *next, *prev;
};

/* NOTE The following are specific to the server, and need to be moved there */

/* a connected, but not necessarily signed-on client */

struct Client {
	time_t connected;	/* time of client connect */
	int dropped;		/* 1 if we told client to leave */
	time_t last;		/* time of last connect */
	struct User *user;	/* pointer to associated user */
};

/* a signed-on user */

struct User {
	int fd;				/* MURGIL file handle */
	char nick[MAX_NICKLEN+1];	/* their assumed name */
	char loginid[MAX_IDLEN+1];	/* their login ID */
	char node[MAX_NODELEN+1];	/* their nodename */
	char realname[MAX_REALLEN+1];	/* their real name */
	time_t ltime;			/* time/date of signon */
	time_t gtime;			/* time user entered group */
	time_t pingtime;		/* time last ping packet sent */
	time_t response;		/* response time */
	time_t oldres;			/* response time */
	char admintries;		/* times attempted ADMIN */
	struct Group *group;		/* their group */
	struct User *next, *prev;	/* next, prev, in group */
};

/* a group */

struct Group {
	char name[MAX_GROUPLEN+1];	/* name of group */
	char topic[MAX_TOPICLEN+1];	/* topic - settable by moderator */
	int restricted;			/* moderator invite needed to enter? */
	struct User *moderator;		/* if not NULL, points to moderator */
	long timeleft;			/* last time moderator was in group */
	int numusers;			/* number of people in group */
	int invitenum;			/* number of invitations */
	struct Group *next, *prev;	/* next, prev, group */
	struct User *head, *tail;	/* next, prev, user */
	struct Namelist *invhead, *invtail;
};
